from mlx import nn
from mlx_vlm import generate, load
from mlx_vlm.prompt_utils import apply_chat_template
from mlx_vlm.utils import load_config

from .constants import DEFAULT_MODEL, DEFAULT_MAX_TOKENS, DEFAULT_TEMPERATURE


class AltTextGenerator:
    """Generate alt-text using MLX=VLM"""

    def __init__(
        self,
        model_name: str = DEFAULT_MODEL,
        max_tokens: int = DEFAULT_MAX_TOKENS,
        temperature: float = DEFAULT_TEMPERATURE,
    ):
        self.model_name = model_name
        self.max_tokens = max_tokens
        self.temperature = temperature
        self.model: None | nn.Module = None
        self.processor = None
        self.config = None

    def _load_model(self):
        """Load the model and processor if not already loaded"""
        if self.model is None:
            self.model, self.processor = load(self.model_name)
            self.config = load_config(self.model_name)
        return self.model, self.processor, self.config

    def generate(self, image: str, prompt: str, verbose: bool = False) -> str:
        """Generate alt-text for the given image"""
        model, processor, config = self._load_model()

        formatted_prompt = apply_chat_template(processor, config, prompt)

        output = generate(
            model,
            processor,  # type: ignore
            formatted_prompt,  # type: ignore
            image=image,
            verbose=verbose,
        )

        return output.strip()



from pathlib import Path

import click

from .constants import *
from .generator import AltTextGenerator


@click.command()
@click.argument(
    "image_path",
    type=click.Path(exists=True, file_okay=True, dir_okay=False, path_type=Path),
)
@click.option(
    "--prompt",
    "-p",
    default="Describe this image in detail for accessibility purposes",
    help="Custom prompt to use for generation",
)
@click.option(
    "--model",
    "-m",
    default=DEFAULT_MODEL,
    help="Model to use for generation, try 'mlx-community/SmolVLM-Instruct-bf16' or 'mlx-community/SmolVLM-256M-Instruct-4bit'",
)
@click.option(
    "--max-tokens",
    type=int,
    default=DEFAULT_MAX_TOKENS,
    help="Maximum tokens to generate",
)
@click.option(
    "--temperature",
    type=float,
    default=DEFAULT_TEMPERATURE,
    help="Temperature for generation",
)
@click.option("--verbose", "-v", is_flag=True, help="Enable verbose output")
def alt_text(image_path, prompt, model, max_tokens, temperature, verbose):
    """Generate alt-text for the provided iamge using local MLX models."""
    generator = AltTextGenerator(model, max_tokens, temperature)
    try:
        result = generator.generate(
            image_path.absolute().as_posix(), prompt, verbose=verbose
        )
        click.echo(result)
    except Exception as e:
        click.echo(f"Error generating alt-text: {e}", err=True)
        raise click.Abort()


def main():
    alt_text()


if __name__ == "__main__":
    main()



from pathlib import Path

import pytest
import mlx_alt_text
from mlx_alt_text import AltTextGenerator

# This model is ~512MB
# https://huggingface.co/mlx-community/SmolVLM-256M-Instruct-bf16
TINY_MODEL = "mlx-community/SmolVLM-256M-Instruct-bf16"
TINY_IMAGE = Path("tests/penguin-image.png")


@pytest.fixture(scope="module")
def alt_text_generator():
    """Creates a single instance of AltTextGenerator."""
    generator = AltTextGenerator(TINY_MODEL)
    assert isinstance(generator, mlx_alt_text.AltTextGenerator)
    return generator


def test_image_prompt(alt_text_generator: AltTextGenerator):
    assert TINY_IMAGE.exists(), f"{TINY_IMAGE.absolute().as_posix()} does not exist"
    response = alt_text_generator.generate(
        TINY_IMAGE.absolute().as_posix(), "Describe the image"
    )
    assert isinstance(response, str)
    assert len(response) > 1
